@@ -1,5 +1,5 @@
 /* gc.h --- Header file for implementation agnostic crypto wrapper API.
- * Copyright (C) 2002-2005, 2007-2008, 2011-2018 Free Software Foundation, Inc.
+ * Copyright (C) 2002-2005, 2007-2008, 2011-2020 Free Software Foundation, Inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published
@@ -16,8 +16,8 @@
  *
  */
 
-#ifndef GC_H
-# define GC_H
+#ifndef _GL_GC_H
+# define _GL_GC_H
 
 /* Get size_t. */
 # include <stddef.h>
@@ -72,6 +72,8 @@ typedef void *gc_hash_handle;
 #define GC_SHA224_DIGEST_SIZE 24
 #define GC_SM3_DIGEST_SIZE 32
 
+#define GC_MAX_DIGEST_SIZE 64
+
 /* Cipher types. */
 enum Gc_cipher
 {
@@ -159,6 +161,8 @@ extern Gc_rc gc_md2 (const void *in, size_t inlen, void *resbuf);
 extern Gc_rc gc_md4 (const void *in, size_t inlen, void *resbuf);
 extern Gc_rc gc_md5 (const void *in, size_t inlen, void *resbuf);
 extern Gc_rc gc_sha1 (const void *in, size_t inlen, void *resbuf);
+extern Gc_rc gc_sha256 (const void *in, size_t inlen, void *resbuf);
+extern Gc_rc gc_sha512 (const void *in, size_t inlen, void *resbuf);
 extern Gc_rc gc_sm3 (const void *in, size_t inlen, void *resbuf);
 extern Gc_rc gc_hmac_md5 (const void *key, size_t keylen,
                           const void *in, size_t inlen, char *resbuf);
@@ -169,17 +173,24 @@ extern Gc_rc gc_hmac_sha256 (const void *key, size_t keylen,
 extern Gc_rc gc_hmac_sha512 (const void *key, size_t keylen,
                              const void *in, size_t inlen, char *resbuf);
 
-/* Derive cryptographic keys from a password P of length PLEN, with
-   salt S of length SLEN, placing the result in pre-allocated buffer
-   DK of length DKLEN.  An iteration count is specified in C, where a
-   larger value means this function take more time (typical iteration
-   counts are 1000-20000).  This function "stretches" the key to be
-   exactly dkLen bytes long.  GC_OK is returned on success, otherwise
-   a Gc_rc error code is returned.  */
+/* Derive cryptographic keys using PKCS#5 PBKDF2 (RFC 2898) from a
+   password P of length PLEN, with salt S of length SLEN, placing the
+   result in pre-allocated buffer DK of length DKLEN.  The PRF is hard
+   coded to be HMAC with HASH.  An iteration count is specified in C
+   (> 0), where a larger value means this function take more time
+   (typical iteration counts are 1000-20000).  This function
+   "stretches" the key to be exactly dkLen bytes long.  GC_OK is
+   returned on success, otherwise a Gc_rc error code is returned.  */
+extern Gc_rc
+gc_pbkdf2_hmac (Gc_hash hash,
+                const char *P, size_t Plen,
+                const char *S, size_t Slen,
+                unsigned int c, char *restrict DK, size_t dkLen);
+
 extern Gc_rc
 gc_pbkdf2_sha1 (const char *P, size_t Plen,
                 const char *S, size_t Slen,
-                unsigned int c, char *DK, size_t dkLen);
+                unsigned int c, char *restrict DK, size_t dkLen);
 
 /*
   TODO:
@@ -321,4 +332,4 @@ gc_pbkdf2_sha1 (const char *P, size_t Plen,
   Simon
  */
 
-#endif /* GC_H */
+#endif /* _GL_GC_H */
