@@ -20,11 +20,7 @@
  *  with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <fcntl.h>
-#include <string.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
+////PULL-HEADERS:
 
 /**
  * Hash a password id into an option search string.
